"""Implementations for adaptive card element types"""

from dataclasses import dataclass, field
from typing import Union, Optional, Any
from dataclasses_json import LetterCase, dataclass_json

from adaptive_cards import actions
from adaptive_cards import utils
import adaptive_cards.card_types as ct

Element = Union["Image", "TextBlock", "Media", "CaptionSource", "RichTextBlock"]


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class CardElement:
    """
    Represents a card element.

    Attributes:
        Element: The element of the card.
        separator: Indicates whether a separator should be displayed before the element.
        spacing: The spacing for the element.
        id: The ID of the element.
        is_visible: Indicates whether the element is visible.
        requires: The requirements for the element.
        height: The height of the element.
    """

    element: Optional[Any | Element] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )
    separator: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.0"))
    spacing: Optional[ct.Spacing] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )
    id: Optional[str] = field(default=None, metadata=utils.get_metadata("1.0")) # pylint: disable=C0103
    is_visible: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.2"))
    requires: Optional[dict[str, str]] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )
    height: Optional[ct.BlockElementHeight] = field(
        default=None, metadata=utils.get_metadata("1.1")
    )


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class TextBlock(CardElement):
    # pylint: disable=too-many-instance-attributes
    """
    Represents a text block card element.

    Inherits from CardElement.

    Attributes:
        text: The text content of the text block.
        type: The type of the card element.
        color: The color of the text block.
        font_type: The font type of the text block.
        horizontal_alignment: The horizontal alignment of the text block.
        is_subtle: Indicates whether the text block has subtle styling.
        max_lines: The maximum number of lines to display for the text block.
        size: The font size of the text block.
        weight: The font weight of the text block.
        wrap: Indicates whether the text should wrap within the text block.
        style: The style of the text block.
    """

    text: str = field(metadata=utils.get_metadata("1.0"))
    type: str = field(default="TextBlock", metadata=utils.get_metadata("1.0"))
    color: Optional[ct.Colors] = field(default=None, metadata=utils.get_metadata("1.0"))
    font_type: Optional[ct.FontType] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )
    horizontal_alignment: Optional[ct.HorizontalAlignment] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )
    is_subtle: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.0"))
    max_lines: Optional[int] = field(default=None, metadata=utils.get_metadata("1.0"))
    size: Optional[ct.FontSize] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )
    weight: Optional[ct.FontWeight] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )
    wrap: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.0"))
    style: Optional[ct.TextBlockStyle] = field(
        default=None, metadata=utils.get_metadata("1.5")
    )


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class Image(CardElement):
    # pylint: disable=too-many-instance-attributes
    """
    Represents an image card element.

    Inherits from CardElement.

    Attributes:
        url: The URL of the image.
        type: The type of the card element.
        alt_text: The alternative text for the image.
        background_color: The background color of the image.
        horizontal_alignment: The horizontal alignment of the image.
        select_action: The select action associated with the image.
        size: The size of the image.
        style: The style of the image.
        width: The width of the image.
    """

    url: str = field(metadata=utils.get_metadata("1.0"))
    type: str = field(default="Image", metadata=utils.get_metadata("1.0"))
    alt_text: Optional[str] = field(default=None, metadata=utils.get_metadata("1.0"))
    background_color: Optional[str] = field(
        default=None, metadata=utils.get_metadata("1.1")
    )
    horizontal_alignment: Optional[ct.HorizontalAlignment] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )
    select_action: Optional[actions.SelectAction] = field(
        default=None, metadata=utils.get_metadata("1.1")
    )
    size: Optional[ct.ImageSize] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )
    style: Optional[ct.ImageStyle] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )
    width: Optional[str] = field(default=None, metadata=utils.get_metadata("1.1"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class Media(CardElement):
    """
    Represents a media card element.

    Inherits from CardElement.

    Attributes:
        type: The type of the card element.
        sources: The list of media sources.
        poster: The poster image URL.
        alt_text: The alternative text for the media.
        caption_sources: The list of caption sources.
    """

    type: str = field(default="Media", metadata=utils.get_metadata("1.1"))
    sources: list["MediaSource"] = field(metadata=utils.get_metadata("1.1"))
    poster: Optional[str] = field(default=None, metadata=utils.get_metadata("1.1"))
    alt_text: Optional[str] = field(default=None, metadata=utils.get_metadata("1.1"))
    caption_sources: Optional[list["CaptionSource"]] = field(
        default=None, metadata=utils.get_metadata("1.6")
    )


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class MediaSource:
    """
    Represents a media source.

    Attributes:
        url: The URL of the media source.
        mime_type: The MIME type of the media source.
    """

    url: str = field(metadata=utils.get_metadata("1.1"))
    mime_type: Optional[str] = field(default=None, metadata=utils.get_metadata("1.1"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class CaptionSource:
    """
    Represents a caption source.

    Attributes:
        mime_type: The MIME type of the caption source.
        url: The URL of the caption source.
        label: The label of the caption source.
    """

    mime_type: str = field(metadata=utils.get_metadata("1.6"))
    url: str = field(metadata=utils.get_metadata("1.6"))
    label: str = field(metadata=utils.get_metadata("1.6"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class RichTextBlock(CardElement):
    """
    Represents a rich text block.

    Inherits from CardElement.

    Attributes:
        inlines: A list of inlines in the rich text block. Each inline can be a string
        or a TextRun object.
        type: The type of the rich text block.
        horizontal_alignment: The horizontal alignment of the rich text block.
    """

    inlines: list[Union[str, "TextRun"]] = field(metadata=utils.get_metadata("1.2"))
    type: str = field(default="RichTextBlock", metadata=utils.get_metadata("1.2"))
    horizontal_alignment: Optional[ct.HorizontalAlignment] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class TextRun:
    # pylint: disable=too-many-instance-attributes
    """
    Represents a text run.

    Attributes:
        text: The text content of the text run.
        type: The type of the text run.
        color: The color of the text run.
        font_type: The font type of the text run.
        highlight: Specifies whether the text run should be highlighted.
        is_subtle: Specifies whether the text run is subtle.
        italic: Specifies whether the text run is italicized.
        select_action: The select action associated with the text run.
        size: The font size of the text run.
        strikethrough: Specifies whether the text run should have a strikethrough effect.
        underline: Specifies whether the text run should be underlined.
        weight: The font weight of the text run.
    """

    text: str = field(metadata=utils.get_metadata("1.2"))
    type: str = field(default="TextRun", metadata=utils.get_metadata("1.2"))
    color: Optional[ct.Colors] = field(default=None, metadata=utils.get_metadata("1.2"))
    font_type: Optional[ct.FontType] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )
    highlight: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.2"))
    is_subtle: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.2"))
    italic: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.2"))
    select_action: Optional[actions.SelectAction] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )
    size: Optional[ct.FontSize] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )
    strikethrough: Optional[bool] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )
    underline: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.3"))
    weight: Optional[ct.FontWeight] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )



"""Implementation of the adaptive card type"""

from dataclasses import dataclass, field
from typing import Any, Literal, Optional, Sequence

from dataclasses_json import LetterCase, dataclass_json

import adaptive_cards.card_types as ct
from adaptive_cards import utils
from adaptive_cards.actions import ActionTypes, SelectAction
from adaptive_cards.containers import ContainerTypes
from adaptive_cards.elements import Element
from adaptive_cards.inputs import InputTypes

SCHEMA: str = "http://adaptivecards.io/schemas/adaptive-card.json"
TYPE: str = "AdaptiveCard"
CardVersion = Literal["1.0", "1.1", "1.2", "1.3", "1.4", "1.5", "1.6"]
VERSION: CardVersion = "1.0"


class AdaptiveCardBuilder:
    """Builder class for creating adaptive cards dynamically"""

    def __init__(self) -> None:
        self.__reset()

    def __reset(self) -> None:
        self.__card = AdaptiveCard()

    def type(self, _type: str) -> "AdaptiveCardBuilder":
        """
        Set type of card

        Args:
            type (str): Type of card

        Returns:
            AdaptiveCardBuilder: Builder object
        """
        self.__card.type = _type
        return self

    def version(self, version: CardVersion) -> "AdaptiveCardBuilder":
        """
        Set card version

        Args:
            version (str): Version of card

        Returns:
            AdaptiveCardBuilder: Builder object
        """
        self.__card.version = version
        return self

    def refresh(self, refresh: ct.Refresh) -> "AdaptiveCardBuilder":
        """
        Set refresh mode

        Args:
            refresh (ct.Refresh): Refresh mode

        Returns:
            AdaptiveCardBuilder: Builder object
        """
        self.__card.refresh = refresh
        return self

    def authentication(
        self, authentication: ct.Authentication
    ) -> "AdaptiveCardBuilder":
        """
        Set authentication mode

        Args:
            authentication (ct.Authentication): Authentication mode

        Returns:
            AdaptiveCardBuilder: Builder object
        """
        self.__card.authentication = authentication
        return self

    def select_action(self, select_action: SelectAction) -> "AdaptiveCardBuilder":
        """
        Set select_action element for card

        Args:
            select_action (SelectAction): Action element when selected

        Returns:
            AdaptiveCardBuilder: Builder object
        """
        self.__card.select_action = select_action
        return self

    def fallback_text(self, fallback_text: str) -> "AdaptiveCardBuilder":
        """
        Set fallback text when rendering fails

        Args:
            fallback_text (str): Text displayed as a fallback

        Returns:
            AdaptiveCardBuilder: Builder object
        """
        self.__card.fallback_text = fallback_text
        return self

    def background_image(
        self, background_image: ct.BackgroundImage
    ) -> "AdaptiveCardBuilder":
        """
        Set background image for card

        Args:
            background_image (ct.BackgroundImage): Image show as background

        Returns:
            AdaptiveCardBuilder: Builder object
        """
        self.__card.background_image = background_image
        return self

    def metadata(self, metadata: ct.Metadata) -> "AdaptiveCardBuilder":
        """
        Set additional metadata for card

        Args:
            metadata (ct.Metadata): Object with addtional metadata

        Returns:
            AdaptiveCardBuilder: Builder object
        """
        self.__card.metadata = metadata
        return self

    def min_height(self, min_height: str) -> "AdaptiveCardBuilder":
        """
        Set minimum card height

        Args:
            min_height (str): Minimum height for card

        Returns:
            AdaptiveCardBuilder: Builder object
        """
        self.__card.min_height = min_height
        return self

    def rtl(self, rtl: bool) -> "AdaptiveCardBuilder":
        """
        Set right-to-left mode for card

        Args:
            rtl (bool): rtl status for card

        Returns:
            AdaptiveCardBuilder: Builder object
        """
        self.__card.rtl = rtl
        return self

    def speak(self, speak: str) -> "AdaptiveCardBuilder":
        """
        Set what should be spoken for card

        Args:
            speak (str): Simple text to be spoken

        Returns:
            AdaptiveCardBuilder: Builder object
        """
        self.__card.speak = speak
        return self

    def lang(self, lang: str) -> "AdaptiveCardBuilder":
        """
        Set card language

        Args:
            lang (str): Language used for card

        Returns:
            AdaptiveCardBuilder: Builder object
        """
        self.__card.lang = lang
        return self

    def vertical_content_alignment(
        self, vertical_content_align: ct.VerticalAlignment
    ) -> "AdaptiveCardBuilder":
        """
        Set vertical alignment for card

        Args:
            vertical_content_align (ct.VerticalAlignment): Vertical alignment of content

        Returns:
            AdaptiveCardBuilder: Builder object
        """
        self.__card.vertical_content_align = vertical_content_align
        return self

    def schema(self, schema: str) -> "AdaptiveCardBuilder":
        """
        Set card schema

        Args:
            schema (str): Card schema

        Returns:
            AdaptiveCardBuilder: Builder object
        """
        self.__card.schema = schema
        return self

    def width(self, width: ct.MSTeamsCardWidth) -> "AdaptiveCardBuilder":
        """
        Set card width for target framework. Please note, changing this property
        will only take affect in MS Teams. Other frameworks will simply ignore this
        field.


        Args:
            width (MSTeamsCardWidth): Width property for card

        Returns:
            AdaptiveCardBuilder: Builder object
        """
        if width == ct.MSTeamsCardWidth.FULL:
            self.__card.msteams = ct.MSTeams(width=width)
            return self

        self.__card.msteams = None
        return self

    def add_item(
        self, item: Element | ContainerTypes | InputTypes
    ) -> "AdaptiveCardBuilder":
        """
        Add single element, container or input to card

        Args:
            item (Element | ContainerTypes | InputTypes): Item to be added to card

        Returns:
            AdaptiveCardBuilder: Builder object
        """
        if self.__card.body is None:
            self.__card.body = []
        self.__card.body.append(item)
        return self

    def add_items(
        self, items: Sequence[Element | ContainerTypes | InputTypes]
    ) -> "AdaptiveCardBuilder":
        """
        Add multiple elements, containers or inputs to card

        Args:
            items (list[Element  |  ContainerTypes  |  InputTypes]): Items to be added to card

        Returns:
            AdaptiveCardBuilder: Builder object
        """
        if self.__card.body is None:
            self.__card.body = []
        for item in items:
            self.add_item(item)

        return self

    def add_action(self, action: ActionTypes) -> "AdaptiveCardBuilder":
        """
        Add single action to card

        Args:
            action (ActionTypes): Action to be added

        Returns:
            AdaptiveCardBuilder: Builder object
        """
        if self.__card.actions is None:
            self.__card.actions = []
        self.__card.actions.append(action)
        return self

    def add_actions(self, actions: list[ActionTypes]) -> "AdaptiveCardBuilder":
        """
        Add multiple actions to card

        Args:
            actions (list[ActionTypes]): Actions to be added

        Returns:
            AdaptiveCardBuilder: Builder object
        """
        if self.__card.actions is None:
            self.__card.actions = []
        for action in actions:
            self.add_action(action)

        return self

    def create(self) -> "AdaptiveCard":
        """
        Create final card object.

        Please note: This method must be called to get a actual card object from the card builder.

        Returns:
            AdaptiveCard: Fully defined apdative card object
        """
        return self.__card


# pylint: disable=too-many-instance-attributes
@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class AdaptiveCard:
    """
    Represents an Adaptive Card.

    Attributes:
        type: The type of the Adaptive Card. Defaults to "AdaptiveCard".
        version: The version of the Adaptive Card.
        schema: The schema of the Adaptive Card.
        refresh: The refresh settings for the card.
        authentication: The authentication settings for the card.
        body: The list of card elements.
        actions: The list of card actions.
        select_action: The select action for the card.
        fallback_text: The fallback text for the card.
        background_image: The background image for the card.
        metadata: The metadata for the card.
        min_height: The minimum height of the card.
        rtl: Whether the card should be displayed right-to-left.
        speak: The speak text for the card.
        lang: The language for the card.
        vertical_content_align: The vertical alignment of the card's content.
        msteams: Set specific properties for MS Teams as the target framework
    """

    type: str = field(default=TYPE, metadata=utils.get_metadata("1.0"))
    version: str = field(default=VERSION, metadata=utils.get_metadata("1.0"))
    schema: str = field(
        default=SCHEMA, metadata=utils.get_metadata("1.0", field_name="$schema")
    )
    refresh: Optional[ct.Refresh] = field(
        default=None, metadata=utils.get_metadata("1.4")
    )
    authentication: Optional[ct.Authentication] = field(
        default=None, metadata=utils.get_metadata("1.4")
    )
    body: Optional[list[Element | ContainerTypes | InputTypes]] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )
    actions: Optional[list[ActionTypes]] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )
    select_action: Optional[SelectAction] = field(
        default=None, metadata=utils.get_metadata("1.1")
    )
    fallback_text: Optional[str] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )
    background_image: Optional[ct.BackgroundImage | str] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )
    metadata: Optional[ct.Metadata] = field(
        default=None, metadata=utils.get_metadata("1.6")
    )
    min_height: Optional[str] = field(default=None, metadata=utils.get_metadata("1.2"))
    rtl: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.5"))
    speak: Optional[str] = field(default=None, metadata=utils.get_metadata("1.0"))
    lang: Optional[str] = field(default=None, metadata=utils.get_metadata("1.0"))
    vertical_content_align: Optional[ct.VerticalAlignment] = field(
        default=None, metadata=utils.get_metadata("1.1")
    )
    msteams: Optional[ct.MSTeams] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )

    @staticmethod
    def new() -> AdaptiveCardBuilder:
        """
        Create a new adaptive card

        Returns:
             AdaptiveCardBuilder: A builder object which allows to set up an adaptive card
                                  step by step.
        """
        return AdaptiveCardBuilder()

    def to_json(self) -> str:
        """
        Converts the full adaptive card schema into a json string.

        Returns:
            str: Adaptive card schema as JSON string.
        """
        return self.to_json()

    def to_dict(self) -> dict[str, Any]:
        """
        Converts the full adaptive card schema into a dictionary.

        Returns:
            str: Adaptive card schema as dictionary.
        """

        return self.to_dict()



"""Validation class for evaluating a cards schema"""

import dataclasses
import json
from abc import ABC, abstractmethod
from dataclasses import dataclass, fields
from enum import Enum, Flag
from pathlib import Path
from typing import Any, Literal

from jsonschema.exceptions import ValidationError
from jsonschema.validators import Draft6Validator

from adaptive_cards.card import AdaptiveCard

MINIMUM_VERSION_KEY: str = "min_version"

SchemaVersion = Literal["1.0", "1.1", "1.2", "1.3", "1.4", "1.5", "1.6"]


class Result(Flag):
    """
    Represents the overall validation result value as a combination of flags.
    """

    SUCCESS = 0
    """validation successful"""

    FAILURE = 1
    """validation failed"""


class ValidationFailure(str, Enum):
    """Validation failure types"""

    EMPTY_CARD = "card body is empty"
    """card doesn't contain any elements"""

    INVALID_FIELD_VERSION = "field version exceeds card version"
    """used elements might not yet be available for the defined card version"""

    INVALID_SCHEMA = "official card schema violated"
    """card structure does not meet the required schema"""

    SIZE_LIMIT_EXCEEDED = "card exceeds the allowed card size for framework"
    """size of card exceeds the defined limit for the target framework"""


class AbstractTargetFramework(ABC):
    """
    Abstract interface representing the AbstractTargetFramework cards can be send to
    """

    def __init__(
        self, name: str, max_card_size_kb: float, schema_version: SchemaVersion
    ) -> None:
        self.__name: str = name
        self.__max_card_size_kb: float = max_card_size_kb
        self.__schema_version: SchemaVersion = schema_version

    def name(self) -> str:
        """
        Return name of target framework

        Returns:
            str: Name of target framework
        """
        return self.__name

    def max_card_size(self) -> float:
        """
        Calculate the maximum allowed card size for the target framework

        Returns:
            float: maximum card size
        """
        return self.__max_card_size_kb

    def schema_version(self) -> SchemaVersion:
        """
        Schema version the card is validated against

        Returns:
            float: schema version
        """
        return self.__schema_version


class BotWebChat(AbstractTargetFramework):
    """Bot WebChat target framework"""

    def __init__(self):
        """"""
        name: str = "BotWebChat"
        max_card_size_kb: float = 40
        schema_version: SchemaVersion = "1.5"
        super().__init__(name, max_card_size_kb, schema_version)


class Outlook(AbstractTargetFramework):
    """Outlook target framework"""

    def __init__(self):
        """"""
        name: str = "Outlook"
        max_card_size_kb: float = 40
        schema_version: SchemaVersion = "1.0"
        super().__init__(name, max_card_size_kb, schema_version)


class MicrosoftTeams(AbstractTargetFramework):
    """MicrosoftTeams target framework"""

    def __init__(self):
        """"""
        name: str = "Microsoft Teams"
        max_card_size_kb: float = 28
        schema_version: SchemaVersion = "1.5"
        super().__init__(name, max_card_size_kb, schema_version)


class CortanaSkills(AbstractTargetFramework):
    """Cortana Skills target framework"""

    def __init__(self):
        """"""
        name: str = "Cortana Skills"
        max_card_size_kb: float = 40
        schema_version: SchemaVersion = "1.0"
        super().__init__(name, max_card_size_kb, schema_version)


class WindowsTimeline(AbstractTargetFramework):
    """Windows Timeline target framework"""

    def __init__(self):
        """"""
        name: str = "Windows Timeline"
        max_card_size_kb: float = 40
        schema_version: SchemaVersion = "1.0"
        super().__init__(name, max_card_size_kb, schema_version)


class CiscoWebExTeams(AbstractTargetFramework):
    """Cisco WebEx Teams target framework"""

    def __init__(self):
        """"""
        name: str = "Cisco WebEx Teams"
        max_card_size_kb: float = 40
        schema_version: SchemaVersion = "1.2"
        super().__init__(name, max_card_size_kb, schema_version)


class VivaConnections(AbstractTargetFramework):
    """Viva Connections target framework"""

    def __init__(self):
        """"""
        name: str = "Viva Connections"
        max_card_size_kb: float = 40
        schema_version: SchemaVersion = "1.2"
        super().__init__(name, max_card_size_kb, schema_version)


class WindowsWidgets(AbstractTargetFramework):
    """Windows Widgets target framework"""

    def __init__(self):
        """"""
        name: str = "WindowsWidgets"
        max_card_size_kb: float = 40
        schema_version: SchemaVersion = "1.6"
        super().__init__(name, max_card_size_kb, schema_version)


class CardValidatorAbstractFactory(ABC):
    """Abstract card validator factory"""

    def __init__(self):
        pass

    @classmethod
    @abstractmethod
    def create_validator_bot(cls) -> "AbstractCardValidator":
        """
        Create card validator for Bot WebChat

        Returns:
            AbstractCardValidator: Card validator for Bot WebChat
        """

    @classmethod
    @abstractmethod
    def create_validator_outlook(cls) -> "AbstractCardValidator":
        """
        Create card validator for outlook

        Returns:
            AbstractCardValidator: Card validator for outlook
        """

    @classmethod
    @abstractmethod
    def create_validator_microsoft_teams(cls) -> "AbstractCardValidator":
        """
        Create card validator for target framework MS Teams

        Returns:
            AbstractCardValidator: Card validator for MS Teams as target framework
        """

    @classmethod
    @abstractmethod
    def create_validator_cortana_skills(cls) -> "AbstractCardValidator":
        """
        Create card validator for Cortana Skills

        Returns:
            AbstractCardValidator: Card validator for Cortana Skills
        """

    @classmethod
    @abstractmethod
    def create_validator_windows_timeline(cls) -> "AbstractCardValidator":
        """
        Create card validator for Windows Timeline

        Returns:
            AbstractCardValidator: Card validator for Windows Timeline
        """

    @classmethod
    @abstractmethod
    def create_validator_cisco_webex_teams(cls) -> "AbstractCardValidator":
        """
        Create card validator for Cisco WebEx Teams

        Returns:
            AbstractCardValidator: Card validator for Cisco WebEx Teams
        """

    @classmethod
    @abstractmethod
    def create_validator_viva_connections(cls) -> "AbstractCardValidator":
        """
        Create card validator for Viva Connections

        Returns:
            AbstractCardValidator: Card validator for Viva Connections
        """

    @classmethod
    @abstractmethod
    def create_validator_windows_widgets(cls) -> "AbstractCardValidator":
        """
        Create card validator for Windows Widgets

        Returns:
            AbstractCardValidator: Card validator for Windows Widgets
        """


class CardValidatorFactory(CardValidatorAbstractFactory):
    """Factory for creating different kinds of card validators"""

    def __init__(self):
        pass

    @classmethod
    def create_validator_bot(cls) -> "CardValidator":
        return CardValidator(BotWebChat())

    @classmethod
    def create_validator_outlook(cls) -> "CardValidator":
        return CardValidator(Outlook())

    @classmethod
    def create_validator_microsoft_teams(cls) -> "CardValidator":
        return CardValidator(MicrosoftTeams())

    @classmethod
    def create_validator_cortana_skills(cls) -> "CardValidator":
        return CardValidator(CortanaSkills())

    @classmethod
    def create_validator_windows_timeline(cls) -> "CardValidator":
        return CardValidator(WindowsTimeline())

    @classmethod
    def create_validator_cisco_webex_teams(cls) -> "CardValidator":
        return CardValidator(CiscoWebExTeams())

    @classmethod
    def create_validator_viva_connections(cls) -> "CardValidator":
        return CardValidator(VivaConnections())

    @classmethod
    def create_validator_windows_widgets(cls) -> "CardValidator":
        return CardValidator(WindowsWidgets())


@dataclass
class Finding:
    """
    Class for storing an individual finding during schema validation
    """

    failure: ValidationFailure
    """Validation failure"""

    message: str
    """Corresponding failure message"""

    message_additional: str = ""
    """Addition information"""


class AbstractCardValidator(ABC):
    """
    Abstract interface for card validators
    """

    def __init__(self):
        pass

    @abstractmethod
    def validate(self, card: AdaptiveCard, debug: bool = True) -> Result:
        """
        Run validation on card.

        Args:
            card (AdaptiveCard): Card to be validated

        Returns:
            Result: Validation result
        """

    @classmethod
    @abstractmethod
    def card_size(cls, card: AdaptiveCard) -> float:
        """
        Get size of card in KB

        Args:
            card (AdaptiveCard): Card the size should be calculated for

        Returns:
            float: Card size in KB
        """

    @abstractmethod
    def details(self) -> list[Finding]:
        """
        Return a list of all individual findings occurred during the last validation. The data
        will last until a new validation is executed.

        Returns:
            list[Finding]: List of findings
        """


class CardValidator(AbstractCardValidator):
    """
    Validator class for checking a cards schema w.r.t. to version numbers of individual fields,
    card size and the overall card structure against the expected schema.
    """

    def __init__(self, target_framework: AbstractTargetFramework) -> None:
        self.__target_framework: AbstractTargetFramework = target_framework
        self.__schema_version: SchemaVersion = target_framework.schema_version()

        self.__card: AdaptiveCard
        self.__item: Any
        self.__findings: list[Finding]
        self.__card_size: float

    def validate(self, card: AdaptiveCard, debug: bool = True) -> Result:
        self.__card = card
        self.__reset()
        self.__validate_card()

        if debug:
            self.__debug()

        return Result.SUCCESS if len(self.__findings) == 0 else Result.FAILURE

    @classmethod
    def card_size(cls, card: AdaptiveCard) -> float:
        return cls.__calculate_card_size(card)

    def details(self) -> list[Finding]:
        return self.__findings

    def __reset(self) -> None:
        self.__findings = []
        self.__card_size: float = 0

    def __validate_card(self) -> None:
        # check whether card body is empty or not
        self.__validate_card_body()

        # Check whether the overall card structure matches the expected schema
        self.__validate_schema()

        # check whether the version requirements are fulfilled for all elements
        self.__validate_version_for_elements(self.__card.body)

        # check whether the card size is within the expected range
        # sizes are derived from the original documentation
        # https://learn.microsoft.com/en-us/microsoftteams/platform/task-modules-and-cards/cards/cards-format?tabs=adaptive-md%2Cdesktop%2Cdesktop1%2Cdesktop2%2Cconnector-html#mention-support-within-adaptive-cards # pylint: disable=line-too-long
        self.__validate_card_size()

    def __validate_card_body(self) -> None:
        if self.__card.body is None:
            self.__findings.append(
                Finding(
                    ValidationFailure.EMPTY_CARD,
                    ValidationFailure.EMPTY_CARD.value,
                )
            )

    @staticmethod
    def __to_list(items: Any | list[Any]) -> list[Any]:
        """
        Put single item into list or keep data as is, if already of type list

        Args:
            items (Any | list[Any]): One or multiple items

        Returns:
            list[Any]: One or multiple items stored as a list
        """
        if not isinstance(items, list):
            items = [items]

        return items

    def __validate_version_for_elements(self, items: Any | list[Any]):
        """
        Recursively check all elements against the overall card version

        Args:
            items (Any): Items to be checked
        """
        # return if no items at all
        if not items:
            return

        items = CardValidator.__to_list(items)

        custom_types: list[Any] = []
        iterables: list[Any] = []

        for item in items:
            self.__item = item
            for field in fields(item):
                value: Any = getattr(item, field.name)

                if value is None:
                    continue

                if isinstance(value, list):
                    iterables.append(value)
                    continue

                if dataclasses.is_dataclass(value):
                    custom_types.append(value)
                    continue

                self.__validate_field_version(
                    field.name, field.metadata.get(MINIMUM_VERSION_KEY)
                )

        for iterable in iterables:
            self.__validate_version_for_elements(iterable)

        for custom_type in custom_types:
            self.__validate_version_for_elements(custom_type)

    @staticmethod
    def __calculate_card_size(card: AdaptiveCard) -> float:
        json_string: bytes = card.to_json().encode("utf-8")
        return len(json_string) / 1024

    def __validate_card_size(self) -> None:
        self.__card_size = CardValidator.__calculate_card_size(self.__card)
        if self.__card_size > self.__target_framework.max_card_size():
            self.__findings.append(
                Finding(
                    ValidationFailure.SIZE_LIMIT_EXCEEDED,
                    ValidationFailure.SIZE_LIMIT_EXCEEDED.value,
                    f"{self.__target_framework.name()} | "
                    f"{self.__target_framework.max_card_size()} KB",
                )
            )

    def __validate_field_version(self, field_name: str, minimum_version: Any) -> None:
        assert minimum_version is not None

        if float(self.__card.version) < float(minimum_version):
            self.__findings.append(
                Finding(
                    ValidationFailure.INVALID_FIELD_VERSION,
                    ValidationFailure.INVALID_FIELD_VERSION.value,
                    f"Field version exceeds card version: {type(self.__item).__name__}"
                    f" | {field_name} | {minimum_version}",
                )
            )

    def __read_schema_file(self) -> dict[str, Any]:
        with open(
            Path(__file__)
            .parent.joinpath("schemas")
            .joinpath(f"schema-{self.__schema_version}.json"),
            "r",
            encoding="utf-8",
        ) as f:  # pylint: disable=C0103
            return json.load(f)

    def __validate_schema(self) -> None:
        schema: dict[str, Any] = self.__read_schema_file()
        try:
            Draft6Validator(schema).validate(json.loads(self.__card.to_json()))

        except ValidationError as ex:
            self.__findings.append(
                Finding(
                    ValidationFailure.INVALID_SCHEMA,
                    ValidationFailure.INVALID_SCHEMA.value,
                    f"{ex.message}",
                )
            )

    def __debug(self):
        for finding in self.__findings:
            print(finding.message, finding.message_additional)



"""Features to send adaptive cards to MS Teams Channels via web hooks.

## Resources

- [Adaptive Cards for Python](https://github.com/dennis6p/adaptive-cards-py)
- [Creating incoming web hooks with workflows](https://support.microsoft.com/en-us/office/create-incoming-webhooks-with-workflows-for-microsoft-teams-8ae491c7-0394-4861-ba59-055e33f75498)
"""  # pylint: disable=line-too-long

from http import HTTPStatus
from typing import Any

import requests
from requests import Response

from adaptive_cards.card import AdaptiveCard


class TeamsClient:
    """Client class for sending adaptive card objects to MS Teams via webhooks"""

    def __init__(self, webhook_url: str) -> None:
        self._webhook_url: str = webhook_url
        """URL the client should communicate with"""

    @staticmethod
    def _create_attachment(card: AdaptiveCard) -> dict:
        """
        Create an attachment from an AdaptiveCard.

        Args:
            card (AdaptiveCard): The AdaptiveCard to create an attachment from.

        Returns:
            dict: A dictionary representing the attachment.
        """
        content: dict[str, Any] = card.to_dict()
        return {
            "contentType": "application/vnd.microsoft.card.adaptive",
            "contentUrl": None,
            "content": content,
        }

    def webhook_url(self) -> str:
        """
        Returns webhook URL the current client has been initialized with

        Returns:
            str: _description_
        """
        return self._webhook_url

    def set_webhook_url(self, webhook_url: str) -> None:
        """
        Update webhook URL the client should use when sending a card

        Args:
            webhook_url (str): Webhook URL
        """
        self._webhook_url = webhook_url

    def send(self, *cards: AdaptiveCard, timeout: int = 1000) -> Response:
        """
        Send the payload of one or multiple cards to a via Microsoft Teams webhook.

        Args:
            cards (tuple[AdaptiveCard, ...]: Card objecs supposed to be sent
            timeout (int): Upper request timeout limit

        Returns:
            Response: Response object from the webhook request.

        Raises:
            ValueError: If no webhook URL or no cards are provided.
            RuntimeError: If the request to the webhook fails.
        """
        if not self._webhook_url:
            raise ValueError("No webhook URL provided.")
        if not cards:
            raise ValueError("No cards provided.")

        headers = {"Content-Type": "application/json"}
        attachments = [self._create_attachment(card) for card in cards]
        payload = {"type": "message", "attachments": attachments}
        response = requests.post(
            self._webhook_url, headers=headers, json=payload, timeout=timeout
        )
        if response.status_code not in (HTTPStatus.OK, HTTPStatus.ACCEPTED):
            raise RuntimeError(
                f"Failed to send card: {response.status_code}, {response.text}"
            )
        return response



"""Implementations for all general card-related types and enums"""

from dataclasses import dataclass, field
from enum import Enum
from typing import Optional

from dataclasses_json import LetterCase, dataclass_json

from adaptive_cards import utils


class MSTeamsCardWidth(str, Enum):
    """
    Enumerates the different fill modes for an image.

    Attributes:
        FULL: The cards' width will be set to full. Occupies all of the
              available horizontal space.
        NONE: The cards' width will be set to None. Default width.
    """

    FULL = "Full"
    DEFAULT = None


class ImageFillMode(str, Enum):
    """
    Enumerates the different fill modes for an image.

    Attributes:
        COVER: The image will be scaled to cover the entire area, possibly cropping parts of the
        image.
        REPEAT_HORIZONTALLY: The image will be repeated horizontally to fill the area.
        REPEAT_VERTICALLY: The image will be repeated vertically to fill the area.
        REPEAT: The image will be repeated both horizontally and vertically to fill the area.
    """

    COVER = "cover"
    REPEAT_HORIZONTALLY = "repeatHorizontally"
    REPEAT_VERTICALLY = "repeatVertically"
    REPEAT = "repeat"


class HorizontalAlignment(str, Enum):
    """
    Enumerates the horizontal alignment options.

    Attributes:
        LEFT: Aligns the content to the left.
        CENTER: Centers the content horizontally.
        RIGHT: Aligns the content to the right.
    """

    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class VerticalAlignment(str, Enum):
    """
    Enumerates the vertical alignment options.

    Attributes:
        TOP: Aligns the content to the top.
        CENTER: Centers the content vertically.
        BOTTOM: Aligns the content to the bottom.
    """

    TOP = "top"
    CENTER = "center"
    BOTTOM = "bottom"


class Colors(str, Enum):
    """
    Enumerates the color options.

    Attributes:
        DEFAULT: The default color.
        DARK: A dark color.
        LIGHT: A light color.
        ACCENT: An accent color.
        GOOD: A color indicating a positive or successful state.
        WARNING: A color indicating a warning or caution state.
        ATTENTION: A color indicating an attention or critical state.
    """

    DEFAULT = "default"
    DARK = "dark"
    LIGHT = "light"
    ACCENT = "accent"
    GOOD = "good"
    WARNING = "warning"
    ATTENTION = "attention"


# class FontType(str, Enum):
class FontType(str, Enum):
    """
    Enumerates the font type options.

    Attributes:
        DEFAULT: The default font type.
        MONOSPACE: A monospace font type.
    """

    DEFAULT = "default"
    MONOSPACE = "monospace"


class FontSize(str, Enum):
    """
    Enumerates the font size options.

    Attributes:
        DEFAULT: The default font size.
        SMALL: A small font size.
        MEDIUM: A medium font size.
        LARGE: A large font size.
        EXTRA_LARGE: An extra large font size.
    """

    DEFAULT = "default"
    SMALL = "small"
    MEDIUM = "medium"
    LARGE = "large"
    EXTRA_LARGE = "extraLarge"


class FontWeight(str, Enum):
    """
    Enumerates the font weight options.

    Attributes:
        DEFAULT: The default font weight.
        LIGHTER: A lighter font weight.
        BOLDER: A bolder font weight.
    """

    DEFAULT = "default"
    LIGHTER = "lighter"
    BOLDER = "bolder"


class TextBlockStyle(str, Enum):
    """
    Enumerates the text block style options.

    Attributes:
        DEFAULT: The default text block style.
        HEADING: A heading style for the text block.
    """

    DEFAULT = "default"
    HEADING = "heading"


class BlockElementHeight(str, Enum):
    """
    Enumerates the block element height options.

    Attributes:
        AUTO: Automatically adjusts the height.
        STRETCH: Stretches the height to fill available space.
    """

    AUTO = "auto"
    STRETCH = "stretch"


class ImageSize(str, Enum):
    """
    Enumerates the image size options.

    Attributes:
        AUTO: Automatically determines the size.
        STRETCH: Stretches the image to fill available space.
        SMALL: A small image size.
        MEDIUM: A medium image size.
        LARGE: A large image size.
    """

    AUTO = "auto"
    STRETCH = "stretch"
    SMALL = "small"
    MEDIUM = "medium"
    LARGE = "large"


class ImageStyle(str, Enum):
    """
    Enumerates the image style options.

    Attributes:
        DEFAULT: The default image style.
        PERSON: An image style for representing a person.
    """

    DEFAULT = "default"
    PERSON = "person"


class ContainerStyle(str, Enum):
    """
    Enumerates the container styles.

    Attributes:
        DEFAULT: The default container style.
        EMPHASIS: An emphasis container style.
        GOOD: A container style indicating a positive status.
        ATTENTION: A container style indicating an attention or important status.
        WARNING: A container style indicating a warning or caution status.
        ACCENT: A container style for accentuated content.
    """

    DEFAULT = "default"
    EMPHASIS = "emphasis"
    GOOD = "good"
    ATTENTION = "attention"
    WARNING = "warning"
    ACCENT = "accent"


class Spacing(str, Enum):
    """
    Enumerates the spacing options.

    Attributes:
        DEFAULT: The default spacing option.
        NONE: No spacing.
        SMALL: Small spacing.
        MEDIUM: Medium spacing.
        LARGE: Large spacing.
        EXTRA_LARGE: Extra large spacing.
        PADDING: Padding spacing.
    """

    DEFAULT = "default"
    NONE = "none"
    SMALL = "small"
    MEDIUM = "medium"
    LARGE = "large"
    EXTRA_LARGE = "extraLarge"
    PADDING = "padding"


class AssociatedInputs(str, Enum):
    """
    Enumerates the associated inputs options.

    Attributes:
        AUTO: Automatically associate inputs.
        NONE: Do not associate inputs.
    """

    AUTO = "auto"
    NONE = "none"


class ActionStyle(str, Enum):
    """
    Enumerates the action styles.

    Attributes:
        DEFAULT: The default action style.
        POSITIVE: A positive or affirmative action style.
        DESTRUCTIVE: A destructive or negative action style.
    """

    DEFAULT = "default"
    POSITIVE = "positive"
    DESTRUCTIVE = "destructive"


class ActionMode(str, Enum):
    """
    Enumerates the action modes.

    Attributes:
        PRIMARY: The primary action mode.
        SECONDARY: The secondary action mode.
    """

    PRIMARY = "primary"
    SECONDARY = "secondary"


class TextInputStyle(str, Enum):
    """
    Enumerates the text input styles.

    Attributes:
        TEXT: The default text input style.
        TEL: The telephone number input style.
        URL: The URL input style.
        EMAIL: The email input style.
        PASSWORD: The password input style.
    """

    TEXT = "text"
    TEL = "tel"
    URL = "url"
    EMAIL = "email"
    PASSWORD = "password"


class ChoiceInputStyle(str, Enum):
    """
    Enumerates the choice input styles.

    Attributes:
        COMPACT: A compact choice input style.
        EXPANDED: An expanded choice input style.
        FILTERED: A filtered choice input style.
    """

    COMPACT = "compact"
    EXPANDED = "expanded"
    FILTERED = "filtered"


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class BackgroundImage:
    """
    Represents the background image properties.

    Attributes:
        uri: The URI of the background image.
        fill_mode: The fill mode of the image.
        horizontal_alignment: The horizontal alignment of the image.
        vertical_alignment: The vertical alignment of the image.
    """

    uri: str = field(metadata=utils.get_metadata("1.0"))
    fill_mode: Optional[ImageFillMode] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )
    horizontal_alignment: Optional[HorizontalAlignment] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )
    vertical_alignment: Optional[VerticalAlignment] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class Refresh:
    """
    Represents the refresh properties.

    Attributes:
        action: The action associated with the refresh.
        expires: The expiration time of the refresh.
        user_ids: The list of user IDs associated with the refresh.
    """

    action: Optional[str] = field(default=None, metadata=utils.get_metadata("1.4"))
    expires: Optional[str] = field(default=None, metadata=utils.get_metadata("1.6"))
    user_ids: Optional[list[str]] = field(
        default=None, metadata=utils.get_metadata("1.4")
    )


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class TokenExchangeResource:
    """
    Represents a token exchange resource.

    Attributes:
        id: The ID of the resource.
        uri: The URI of the resource.
        provider_id: The provider ID associated with the resource.
    """

    id: str = field(default="", metadata=utils.get_metadata("1.4"))  # pylint: disable=C0103
    uri: str = field(default="", metadata=utils.get_metadata("1.4"))
    provider_id: str = field(default="", metadata=utils.get_metadata("1.4"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class AuthCardButtons:
    """
    Represents buttons used in an authentication card.

    Attributes:
        type: The type of the button.
        value: The value associated with the button.
        title: The title of the button.
        image: The image URL of the button.
    """

    type: str = field(default="", metadata=utils.get_metadata("1.4"))
    value: str = field(default="", metadata=utils.get_metadata("1.4"))
    title: Optional[str] = field(default=None, metadata=utils.get_metadata("1.4"))
    image: Optional[str] = field(default=None, metadata=utils.get_metadata("1.4"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class Authentication:
    """
    Represents authentication properties.

    Attributes:
        text: The authentication text.
        connection_name: The connection name.
        token_exchange_resource: The token exchange resource.
        buttons: The authentication buttons.
    """

    text: Optional[str] = field(default=None, metadata=utils.get_metadata("1.4"))
    connection_name: Optional[str] = field(
        default=None, metadata=utils.get_metadata("1.4")
    )
    token_exchange_resource: Optional[TokenExchangeResource] = field(
        default=None, metadata=utils.get_metadata("1.4")
    )
    buttons: Optional[AuthCardButtons] = field(
        default=None, metadata=utils.get_metadata("1.4")
    )


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class Metadata:
    """
    Represents metadata properties.

    Attributes:
        web_url: The web URL.
    """

    web_url: Optional[str] = field(default=None, metadata=utils.get_metadata("1.6"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class MSTeams:
    """
    Represents specific properties for MS Teams as the target framework.

    Attributes:
        width: The total horizontal space an adaptive cards is allowed to occupy
               when posted to MS Teams. Defaults to "None".
    """

    width: Optional[MSTeamsCardWidth] = field(
        default=MSTeamsCardWidth.DEFAULT, metadata=utils.get_metadata("1.0")
    )



"""
This package provides small components for building adaptive cards compliant to 
the current interface definite with Python. 

[Schema Explorer](https://adaptivecards.io/explorer/)

This `__init__.py` file exposes key components of the package for easier access:

By importing the package, the following modulesare directly available:
    - [actions, card_types, card, client, containers, elements, inputs, utils, validation]

This module also initializes the package and ensures that any necessary 
configuration or setup is performed.
"""

from adaptive_cards.actions import *
from adaptive_cards.card_types import *
from adaptive_cards.card import *
from adaptive_cards.client import *
from adaptive_cards.containers import *
from adaptive_cards.elements import *
from adaptive_cards.inputs import *
from adaptive_cards.utils import *
from adaptive_cards.validation import *



"""Tests for validation module"""

import unittest
from dataclasses import dataclass, field

from dataclasses_json import dataclass_json

import adaptive_cards.card_types as types
from adaptive_cards import (
    AdaptiveCard,
    TextBlock,
    utils,
)
from adaptive_cards.validation import (
    CardValidator,
    CardValidatorFactory,
    Result,
    ValidationFailure,
)


class TestAdaptiveCardValidation(unittest.TestCase):
    """Test class for Adaptive Card validaiton"""

    def test_card_validator_ms_teams_validate_success(self) -> None:
        """Test validation for ms teams"""
        validator: CardValidator = (
            CardValidatorFactory.create_validator_microsoft_teams()
        )
        card: AdaptiveCard = (
            AdaptiveCard.new().add_item(TextBlock(text="Test Card")).create()
        )
        self.assertEqual(validator.validate(card), Result.SUCCESS)
        self.assertEqual(len(validator.details()), 0)

    def test_validate_failure_empty_body(self) -> None:
        """Test validation for ms teams"""
        validator: CardValidator = (
            CardValidatorFactory.create_validator_microsoft_teams()
        )
        card: AdaptiveCard = AdaptiveCard.new().create()
        self.assertEqual(validator.validate(card), Result.FAILURE)
        self.assertEqual(len(validator.details()), 1)
        self.assertEqual(validator.details()[0].failure, ValidationFailure.EMPTY_CARD)

    def test_validate_failure_invalid_field_version(self) -> None:
        """Test validation for ms teams"""
        validator: CardValidator = (
            CardValidatorFactory.create_validator_microsoft_teams()
        )
        card: AdaptiveCard = (
            AdaptiveCard.new()
            .version("1.0")
            .add_item(TextBlock(text="Test Card", font_type=types.FontType.MONOSPACE))
            .create()
        )
        self.assertEqual(validator.validate(card), Result.FAILURE)
        self.assertEqual(len(validator.details()), 1)
        self.assertEqual(
            validator.details()[0].failure, ValidationFailure.INVALID_FIELD_VERSION
        )

    def test_validate_failure_invalid_schema(self) -> None:
        """Test validation for ms teams"""
        validator: CardValidator = (
            CardValidatorFactory.create_validator_microsoft_teams()
        )

        @dataclass_json
        @dataclass
        class InvalidClass:
            """Invalid class"""

            some_field: int | None = field(
                default=None, metadata=utils.get_metadata("1.0")
            )

        card: AdaptiveCard = (
            AdaptiveCard.new().version("1.0").add_item(InvalidClass(1)).create()  #
        )
        self.assertEqual(validator.validate(card), Result.FAILURE)
        self.assertEqual(len(validator.details()), 1)
        self.assertEqual(
            validator.details()[0].failure, ValidationFailure.INVALID_SCHEMA
        )

    def test_validate_failure_size_limit_exceeded(self) -> None:
        """Test validation for ms teams"""
        validator: CardValidator = (
            CardValidatorFactory.create_validator_microsoft_teams()
        )

        card: AdaptiveCard = (
            AdaptiveCard.new()
            .version("1.0")
            .add_items([TextBlock(text="TestCard") for i in range(660)])
            .create()  #
        )

        self.assertTrue(validator.card_size(card) < 28)
        self.assertEqual(validator.validate(card), Result.SUCCESS)

        card = (
            AdaptiveCard.new()
            .version("1.0")
            .add_items([TextBlock(text="TestCard") for i in range(670)])
            .create()  #
        )
        self.assertTrue(validator.card_size(card) > 28)
        self.assertEqual(validator.validate(card), Result.FAILURE)
        self.assertEqual(len(validator.details()), 1)
        self.assertEqual(
            validator.details()[0].failure, ValidationFailure.SIZE_LIMIT_EXCEEDED
        )


if __name__ == "__main__":
    unittest.main()



"""Implementations for all adaptive card container types"""

from dataclasses import dataclass, field
from typing import Optional, Union

from dataclasses_json import LetterCase, dataclass_json

import adaptive_cards.actions as action
import adaptive_cards.card_types as ct
from adaptive_cards import elements, inputs, utils

ContainerTypes = Union[
    "ActionSet", "Container", "ColumnSet", "FactSet", "ImageSet", "Table"
]


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class ContainerBase:
    """
    The ContainerBase class represents a base container for elements with various properties.

    Attributes:
        fallback: The fallback element, if any, to be displayed when the container
        cannot be rendered.
        separator: Determines whether a separator should be shown above the container.
        spacing: The spacing style to be applied within the container.
        id: The unique identifier of the container.
        is_visible: Determines whether the container is visible.
        requires: A dictionary of requirements that must be satisfied for the container
        to be displayed.
        height: The height style to be applied to the container.
    """

    fallback: Optional[elements.Element | action.ActionTypes | inputs.InputTypes] = (
        field(default=None, metadata=utils.get_metadata("1.2"))
    )
    separator: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.2"))
    spacing: Optional[ct.Spacing] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )
    # pylint: disable=C0103
    id: Optional[str] = field(default=None, metadata=utils.get_metadata("1.2"))
    is_visible: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.2"))
    requires: Optional[dict[str, str]] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )
    height: Optional[ct.BlockElementHeight] = field(
        default=None, metadata=utils.get_metadata("1.1")
    )


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class ActionSet(ContainerBase):
    """Represents an action set, a container for a list of actions.

    Inherits from ContainerBase.

    Attributes:
        actions: A list of actions in the action set.
        type: The type of the action set. Defaults to "ActionSet".
    """

    actions: list[action.ActionTypes] = field(metadata=utils.get_metadata("1.2"))
    type: str = field(default="ActionSet", metadata=utils.get_metadata("1.2"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class Container(ContainerBase):
    # pylint: disable=too-many-instance-attributes
    """Represents a container for elements with various properties.

    Inherits from ContainerBase.

    Attributes:
        items: A list of elements, sub-containers, or input elements contained within the container.
        type: The type of the container. Defaults to "Container".
        select_action: An optional select action associated with the container.
        style: The style of the container.
        vertical_content_alignment: The vertical alignment of the container's content.
        bleed: Determines whether the container bleeds beyond its boundary.
        background_image: The background image of the container.
        min_height: The minimum height of the container.
        rtl: Determines whether the container's content is displayed right-to-left.
    """

    items: list[elements.Element | ContainerTypes | inputs.InputTypes] = field(
        metadata=utils.get_metadata("1.0")
    )
    type: str = field(default="Container", metadata=utils.get_metadata("1.0"))
    select_action: Optional[action.SelectAction] = field(
        default=None, metadata=utils.get_metadata("1.1")
    )
    style: Optional[ct.ContainerStyle] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )
    vertical_content_alignment: Optional[ct.VerticalAlignment] = field(
        default=None, metadata=utils.get_metadata("1.1")
    )
    bleed: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.2"))
    background_image: Optional[ct.BackgroundImage | str] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )
    min_height: Optional[str] = field(default=None, metadata=utils.get_metadata("1.2"))
    rtl: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.5"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class ColumnSet(ContainerBase):
    """Represents a set of columns within a container.

    Inherits from ContainerBase.

    Attributes:
        type: The type of the column set. Defaults to "ColumnSet".
        columns: An optional list of Column objects within the column set.
        select_action: An optional select action associated with the column set.
        style: The style of the column set.
        bleed: Determines whether the column set bleeds beyond its boundary.
        min_height: The minimum height of the column set.
        horizontal_alignment: The horizontal alignment of the column set.
    """

    type: str = field(default="ColumnSet", metadata=utils.get_metadata("1.0"))
    columns: Optional[list["Column"]] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )
    select_action: Optional[action.SelectAction] = field(
        default=None, metadata=utils.get_metadata("1.1")
    )
    style: Optional[ct.ContainerStyle] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )
    bleed: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.2"))
    min_height: Optional[str] = field(default=None, metadata=utils.get_metadata("1.2"))
    horizontal_alignment: Optional[ct.HorizontalAlignment] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class Column(ContainerBase):
    # pylint: disable=too-many-instance-attributes
    """Represents a column within a container.

    Inherits from ContainerBase.

    Attributes:
        type: The type of the column. Defaults to "Column".
        items: An optional list of elements contained within the column.
        background_image: The background image of the column.
        bleed: Determines whether the column bleeds beyond its boundary.
        min_height: The minimum height of the column.
        rtl: Determines whether the column's content is displayed right-to-left.
        separator: Determines whether a separator should be shown above the column.
        spacing: The spacing style to be applied within the column.
        select_action: An optional select action associated with the column.
        style: The style of the column.
        vertical_content_alignment: The vertical alignment of the column's content.
        width: The width of the column.
    """

    type: str = field(default="Column", metadata=utils.get_metadata("1.0"))
    items: Optional[list[elements.Element | ContainerTypes | inputs.Input]] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )
    background_image: Optional[ct.BackgroundImage | str] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )
    bleed: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.2"))
    min_height: Optional[str] = field(default=None, metadata=utils.get_metadata("1.2"))
    rtl: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.5"))
    separator: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.0"))
    spacing: Optional[ct.Spacing] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )
    select_action: Optional[action.SelectAction] = field(
        default=None, metadata=utils.get_metadata("1.1")
    )
    style: Optional[ct.ContainerStyle] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )
    vertical_content_alignment: Optional[ct.VerticalAlignment] = field(
        default=None, metadata=utils.get_metadata("1.1")
    )
    width: Optional[str | int] = field(default=None, metadata=utils.get_metadata("1.0"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class FactSet(ContainerBase):
    """Represents a set of facts within a container.

    Inherits from ContainerBase.

    Attributes:
        facts: A list of Fact objects within the fact set.
        type: The type of the fact set. Defaults to "FactSet".
    """

    facts: list["Fact"] = field(metadata=utils.get_metadata("1.0"))
    type: str = field(default="FactSet", metadata=utils.get_metadata("1.0"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class Fact:
    """Represents a fact.

    Attributes:
        title: The title of the fact.
        value: The value of the fact.
    """

    title: str = field(metadata=utils.get_metadata("1.0"))
    value: str = field(metadata=utils.get_metadata("1.0"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class ImageSet(ContainerBase):
    """Represents a set of images within a container.

    Inherits from ContainerBase.

    Attributes:
        images: A list of Image objects within the image set.
        type: The type of the image set. Defaults to "ImageSet".
        image_size: The size of the images within the image set.
    """

    images: list[elements.Image] = field(metadata=utils.get_metadata("1.0"))
    type: str = field(default="ImageSet", metadata=utils.get_metadata("1.2"))
    image_size: Optional[ct.ImageSize] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class TableColumnDefinition:
    """Represents a definition for a table column.

    Attributes:
        horizontal_cell_content_alignment: The horizontal alignment of cell content.
        vertical_cell_content_alignment: The vertical alignment of cell content.
        width: The width of the table column.
    """

    horizontal_cell_content_alignment: Optional[ct.HorizontalAlignment] = field(
        default=None, metadata=utils.get_metadata("1.5")
    )
    vertical_cell_content_alignment: Optional[ct.VerticalAlignment] = field(
        default=None, metadata=utils.get_metadata("1.5")
    )
    width: Optional[str | int] = field(default=None, metadata=utils.get_metadata("1.5"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class TableRow:
    """Represents a row within a table.

    Attributes:
        cells: The cells within the table row.
        horizontal_cell_content_alignment: The horizontal alignment of cell content.
        vertical_cell_content_alignment: The vertical alignment of cell content.
        style: The style of the table row.
    """

    type: str = field(default="TableRow", metadata=utils.get_metadata("1.5"))
    cells: Optional["TableCell"] = field(
        default=None, metadata=utils.get_metadata("1.5")
    )
    horizontal_cell_content_alignment: Optional[ct.HorizontalAlignment] = field(
        default=None, metadata=utils.get_metadata("1.5")
    )
    vertical_cell_content_alignment: Optional[ct.VerticalAlignment] = field(
        default=None, metadata=utils.get_metadata("1.5")
    )
    style: Optional[str] = field(default=None, metadata=utils.get_metadata("1.5"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class Table(ContainerBase):
    # pylint: disable=too-many-instance-attributes
    """Represents a table within a container.

    Inherits from ContainerBase.

    Attributes:
        type: The type of the table. Defaults to "Table".
        columns: The column definitions of the table.
        rows: The rows of the table.
        first_row_as_header: Whether the first row should be treated as a header.
        show_grid_lines: Whether to show grid lines in the table.
        grid_style: The style of the table grid.
        horizontal_cell_content_alignment: The horizontal alignment of cell content.
        vertical_cell_content_alignment: The vertical alignment of cell content.
    """

    type: str = field(default="Table", metadata=utils.get_metadata("1.5"))
    columns: Optional[list[TableColumnDefinition]] = field(
        default=None, metadata=utils.get_metadata("1.5")
    )
    rows: Optional[list[TableRow]] = field(
        default=None, metadata=utils.get_metadata("1.5")
    )
    first_row_as_header: Optional[bool] = field(
        default=None, metadata=utils.get_metadata("1.5")
    )
    show_grid_lines: Optional[bool] = field(
        default=None, metadata=utils.get_metadata("1.5")
    )
    grid_style: Optional[ct.ContainerStyle] = field(
        default=None, metadata=utils.get_metadata("1.5")
    )
    horizontal_cell_content_alignment: Optional[ct.HorizontalAlignment] = field(
        default=None, metadata=utils.get_metadata("1.5")
    )
    vertical_cell_content_alignment: Optional[ct.VerticalAlignment] = field(
        default=None, metadata=utils.get_metadata("1.5")
    )


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class TableCell:
    # pylint: disable=too-many-instance-attributes
    """Represents a cell within a table.

    Attributes:
        items: The elements within the cell.
        select_action: The action to perform when the cell is selected.
        style: The style of the cell.
        vertical_content_alignment: The vertical alignment of cell content.
        bleed: Whether the cell should bleed beyond its boundaries.
        background_image: The background image of the cell.
        min_height: The minimum height of the cell.
        rtl: Whether the cell should be rendered in right-to-left direction.
    """

    type: str = field(default="TableCell", metadata=utils.get_metadata("1.5"))
    items: list[elements.Element] = field(metadata=utils.get_metadata("1.5"))
    select_action: Optional[action.SelectAction] = field(
        default=None, metadata=utils.get_metadata("1.1")
    )
    style: Optional[ct.ContainerStyle] = field(
        default=None, metadata=utils.get_metadata("1.5")
    )
    vertical_content_alignment: Optional[ct.VerticalAlignment] = field(
        default=None, metadata=utils.get_metadata("1.1")
    )
    bleed: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.2"))
    background_image: Optional[ct.BackgroundImage | str] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )
    min_height: Optional[str] = field(default=None, metadata=utils.get_metadata("1.2"))
    rtl: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.5"))



"""
Utility functions for library
"""

from typing import Any

from dataclasses_json import config


def is_none(item: Any) -> bool:
    """
    Check whether a given item is initialized or None

    Args:
        item (Any): Item to be checked

    Returns:
        bool: true if item is None
    """
    return item is None


def get_metadata(min_version: str, field_name: str | None = None) -> dict[str, Any]:
    """
    Get default metadata information for dataclass field

    Args:
        min_version (str): Minimum version number of the field the result will
                           be appliead to.

    Returns:
        dict[str, Any]: Metadata information
    """
    return config(exclude=is_none, field_name=field_name) | {"min_version": min_version}



"""Example: simple card"""

from requests import Response

import adaptive_cards.card_types as types
from adaptive_cards.card import AdaptiveCard
from adaptive_cards.client import TeamsClient
from adaptive_cards.elements import TextBlock
from adaptive_cards.validation import CardValidator, CardValidatorFactory, Result

text_block: TextBlock = TextBlock(
    text="It's your second card",
    color=types.Colors.ACCENT,
    size=types.FontSize.EXTRA_LARGE,
    horizontal_alignment=types.HorizontalAlignment.CENTER,
)

# build card
version: str = "1.4"
card: AdaptiveCard = AdaptiveCard.new().version(version).add_item(text_block).create()

# validate card
validator: CardValidator = CardValidatorFactory.create_validator_microsoft_teams()
result: Result = validator.validate(card)

print(f"Validation was successful: {result == Result.SUCCESS}")

# send card
webhook_url: str = "YOUR-URL"
client: TeamsClient = TeamsClient(webhook_url)
response: Response = client.send(card)



"""Implementations for all adaptive card action types"""

from dataclasses import dataclass, field
from typing import Any, Optional, Union

from dataclasses_json import LetterCase, dataclass_json

import adaptive_cards.card_types as ct
from adaptive_cards import utils

ActionTypes = Union[
    "ActionOpenUrl",
    "ActionSubmit",
    "ActionShowCard",
    "ActionToggleVisibility",
    "ActionExecute",
]
SelectAction = Union[
    "ActionExecute", "ActionOpenUrl", "ActionSubmit", "ActionToggleVisibility"
]


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class Action:
    # pylint: disable=too-many-instance-attributes
    """
    Represents an action that can be performed.

    Attributes:
        title: An optional string representing the title of the action.
        icon_url: An optional string representing the URL of the icon associated with the action.
        id: An optional string representing the ID of the action.
        style: An optional ActionStyle enum value representing the style of the action.
        fallback: An optional fallback ActionTypes object representing the fallback action to be
        performed.
        tooltip: An optional string representing the tooltip text for the action.
        is_enabled: An optional boolean indicating whether the action is enabled or disabled.
        mode: An optional ActionMode enum value representing the mode of the action.
        requires: An optional dictionary mapping string keys to string values representing the
        requirements for the action.
    """

    title: Optional[str] = field(default=None, metadata=utils.get_metadata("1.0"))
    icon_url: Optional[str] = field(default=None, metadata=utils.get_metadata("1.1"))
    id: Optional[str] = field(default=None, metadata=utils.get_metadata("1.0"))  # pylint: disable=C0103
    style: Optional[ct.ActionStyle] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )
    fallback: Optional[ActionTypes] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )
    tooltip: Optional[str] = field(default=None, metadata=utils.get_metadata("1.5"))
    is_enabled: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.5"))
    mode: Optional[ct.ActionMode] = field(
        default=ct.ActionMode.PRIMARY, metadata=utils.get_metadata("1.5")
    )
    requires: Optional[dict[str, str]] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class ActionOpenUrl(Action):
    """
    Represents an action to open a URL.

    Inherits from Action

    Attributes:
        url: The URL to be opened.
        type: The type of the action. Default is "Action.OpenUrl".
    """

    url: str = field(metadata=utils.get_metadata("1.0"))
    type: str = field(default="Action.OpenUrl", metadata=utils.get_metadata("1.0"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class ActionSubmit(Action):
    """
    Represents an action to submit data.

    Inherits from Action.

    Attributes:
        type: The type of the action. Default is "Action.Submit".
        data: Optional data associated with the action.
        associated_inputs: Optional associated inputs for the action.
    """

    type: str = field(default="Action.Submit", metadata=utils.get_metadata("1.0"))
    data: Optional[str | Any] = field(default=None, metadata=utils.get_metadata("1.0"))
    associated_inputs: Optional[ct.AssociatedInputs] = field(
        default=None, metadata=utils.get_metadata("1.3")
    )


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class ActionShowCard(Action):
    """
    Represents an action to show a card.

    Inherits from Action.

    Attributes:
        type: The type of the action. Default is "Action.ShowCard".
        card: Optional card to show.
    """

    type: str = field(default="Action.ShowCard", metadata=utils.get_metadata("1.0"))
    card: Optional[Any] = field(default=None, metadata=utils.get_metadata("1.0"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class TargetElement:
    """
    Represents a target element.

    Attributes:
        element_id: The ID of the target element.
        is_visible: Optional flag indicating the visibility of the target element.
    """

    element_id: str = field(metadata=utils.get_metadata("1.0"))
    is_visible: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.0"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class ActionToggleVisibility(Action):
    """
    Represents an action that toggles the visibility of target elements.

    Inherits from Action.

    Attributes:
        target_elements: A list of TargetElement objects representing the target elements to toggle.
        type: The type of the action, set to "Action.ToggleVisibility".
    """

    target_elements: list[TargetElement] = field(metadata=utils.get_metadata("1.2"))
    type: str = field(
        default="Action.ToggleVisibility", metadata=utils.get_metadata("1.2")
    )


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class ActionExecute(Action):
    """
    Represents an action that executes a command or performs an action.

    Inherits from Action.

    Attributes:
        type: The type of the action, set to "Action.ShowCard".
        verb: An optional string representing the verb of the action.
        data: An optional string or Any type representing additional data associated
        with the action.
        associated_inputs: An optional AssociatedInputs object representing associated
        inputs for the action.
    """

    type: str = field(default="Action.ShowCard", metadata=utils.get_metadata("1.4"))
    verb: Optional[str] = field(default=None, metadata=utils.get_metadata("1.4"))
    data: Optional[str | Any] = field(default=None, metadata=utils.get_metadata("1.4"))
    associated_inputs: Optional[ct.AssociatedInputs] = field(
        default=None, metadata=utils.get_metadata("1.4")
    )



"""Implementations for adaptive card input types"""

from dataclasses import dataclass, field
from typing import Union, Optional
from dataclasses_json import dataclass_json, LetterCase
from adaptive_cards import utils
import adaptive_cards.card_types as ct
from adaptive_cards import actions

InputTypes = Union[
    "InputText",
    "InputNumber",
    "InputDate",
    "InputTime",
    "InputToggle",
    "InputChoiceSet",
]


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class Input:
    # pylint: disable=too-many-instance-attributes
    """
    Represents an input.

    Attributes:
        error_message: The error message for the input.
        is_required: Specifies whether the input is required.
        label: The label of the input.
        fallback: The fallback input.
        height: The height of the input.
        separator: Specifies whether a separator should be displayed before the input.
        spacing: The spacing of the input.
        is_visible: Specifies whether the input is visible.
        requires: The requirements for the input.
    """

    error_message: Optional[str] = field(
        default=None, metadata=utils.get_metadata("1.3")
    )
    is_required: Optional[bool] = field(
        default=None, metadata=utils.get_metadata("1.3")
    )
    label: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.3"))
    fallback: Optional[InputTypes] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )
    height: Optional[ct.BlockElementHeight] = field(
        default=None, metadata=utils.get_metadata("1.1")
    )
    separator: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.0"))
    spacing: Optional[ct.Spacing] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )
    is_visible: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.2"))
    requires: Optional[dict[str, str]] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class InputText(Input):
    # pylint: disable=too-many-instance-attributes
    """
    Represents a text input.

    Inherits from Input.

    Attributes:
        id: The ID of the input.
        type: The type of the input.
        is_multiline: Specifies whether the input supports multiline text.
        max_length: The maximum length of the input text.
        placeholder: The placeholder text for the input.
        regex: The regular expression pattern for validating the input text.
        style: The style of the text input.
        inline_action: The inline action associated with the input.
        value: The initial value of the input.
    """

    id: str = field(metadata=utils.get_metadata("1.0"))  # pylint: disable=C0103
    type: str = field(default="Input.Text", metadata=utils.get_metadata("1.0"))
    is_multiline: Optional[bool] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )
    max_length: Optional[int] = field(default=None, metadata=utils.get_metadata("1.0"))
    placeholder: Optional[str] = field(default=None, metadata=utils.get_metadata("1.0"))
    regex: Optional[str] = field(default=None, metadata=utils.get_metadata("1.3"))
    style: Optional[ct.TextInputStyle] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )
    inline_action: Optional[actions.SelectAction] = field(
        default=None, metadata=utils.get_metadata("1.2")
    )
    value: Optional[str] = field(default=None, metadata=utils.get_metadata("1.0"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class InputNumber(Input):
    """
    Represents an input field for numerical values.

    Inherits from Input.

    Attributes:
        id: The ID of the input.
        type: The type of the input, which is "Input.Number".
        max: The maximum value allowed for the input. Optional.
        min: The minimum value allowed for the input. Optional.
        placeholder: The placeholder text for the input. Optional.
        value: The initial value of the input. Optional.
    """

    id: str = field(metadata=utils.get_metadata("1.0"))  # pylint: disable=C0103
    type: str = field(default="Input.Number", metadata=utils.get_metadata("1.0"))
    max: Optional[int] = field(default=None, metadata=utils.get_metadata("1.0"))
    max: Optional[int] = field(default=None, metadata=utils.get_metadata("1.0"))
    placeholder: Optional[str] = field(default=None, metadata=utils.get_metadata("1.0"))
    value: Optional[int] = field(default=None, metadata=utils.get_metadata("1.0"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class InputDate(Input):
    """
    Represents an input field for date values.

    Inherits from Input.

    Attributes:
        id: The ID of the input.
        type: The type of the input, which is "Input.Date".
        max: The maximum date allowed for the input. Optional.
        placeholder: The placeholder text for the input. Optional.
        value: The initial value of the input. Optional.
    """

    id: str = field(metadata=utils.get_metadata("1.0"))  # pylint: disable=C0103
    type: str = field(default="Input.Date", metadata=utils.get_metadata("1.0"))
    max: Optional[str] = field(default=None, metadata=utils.get_metadata("1.0"))
    placeholder: Optional[str] = field(default=None, metadata=utils.get_metadata("1.0"))
    value: Optional[str] = field(default=None, metadata=utils.get_metadata("1.0"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class InputTime(Input):
    """
    Represents an input field for time values.

    Inherits from Input.

    Attributes:
        id: The ID of the input.
        type: The type of the input, which is "Input.Time".
        max: The maximum time allowed for the input. Optional.
        placeholder: The placeholder text for the input. Optional.
        value: The initial value of the input. Optional.
    """

    id: str = field(metadata=utils.get_metadata("1.0"))  # pylint: disable=C0103
    type: str = field(default="Input.Time", metadata=utils.get_metadata("1.0"))
    max: Optional[str] = field(default=None, metadata=utils.get_metadata("1.0"))
    max: Optional[str] = field(default=None, metadata=utils.get_metadata("1.0"))
    placeholder: Optional[str] = field(default=None, metadata=utils.get_metadata("1.0"))
    value: Optional[str] = field(default=None, metadata=utils.get_metadata("1.0"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class InputToggle(Input):
    """
    Represents a toggle input field.

    Inherits from Input.

    Attributes:
        id: The ID of the input.
        title: The title or label for the input.
        type: The type of the input, which is "Input.Toggle".
        value: The initial value of the input. Optional.
        value_off: The value when the toggle is turned off. Optional.
        value_on: The value when the toggle is turned on. Optional.
        wrap: Indicates whether the input should wrap to the next line if needed. Optional.
    """

    id: str = field(metadata=utils.get_metadata("1.0"))  # pylint: disable=C0103
    title: str = field(metadata=utils.get_metadata("1.0"))
    type: str = field(default="Input.Toggle", metadata=utils.get_metadata("1.0"))
    value: Optional[str] = field(default=None, metadata=utils.get_metadata("1.0"))
    value_off: Optional[str] = field(default=None, metadata=utils.get_metadata("1.0"))
    value_on: Optional[str] = field(default=None, metadata=utils.get_metadata("1.0"))
    wrap: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.2"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class InputChoiceSet(Input):
    # pylint: disable=too-many-instance-attributes
    """
    Represents a choice set input field.

    Inherits from Input.

    Attributes:
        id: The ID of the input.
        type: The type of the input, which is "Input.ChoiceSet".
        choices: The list of choices for the input. Optional.
        is_multi_select: Indicates whether multiple choices can be selected. Optional.
        style: The style of the choice input. Optional.
        value: The initial value of the input. Optional.
        placeholder: The placeholder text for the input. Optional.
        wrap: Indicates whether the input should wrap to the next line if needed. Optional.
    """

    id: str = field(metadata=utils.get_metadata("1.0"))  # pylint: disable=C0103
    type: str = field(default="Input.ChoiceSet", metadata=utils.get_metadata("1.0"))
    choices: Optional[list["InputChoice"]] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )
    is_multi_select: Optional[bool] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )
    style: Optional[ct.ChoiceInputStyle] = field(
        default=None, metadata=utils.get_metadata("1.0")
    )
    value: Optional[str] = field(default=None, metadata=utils.get_metadata("1.0"))
    placeholder: Optional[str] = field(default=None, metadata=utils.get_metadata("1.0"))
    wrap: Optional[bool] = field(default=None, metadata=utils.get_metadata("1.2"))


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass(kw_only=True)
class InputChoice:
    """
    Represents a choice within an input choice set.

    Attributes:
        title: The title or display text of the choice.
        value: The value associated with the choice.
    """

    title: str = field(metadata=utils.get_metadata("1.0"))
    value: str = field(metadata=utils.get_metadata("1.0"))
